const utils = require('../utils');

/**
 * Attempt to load package.json within the current directory.
 * @param {array} results The output generated by running ESLint.
 * @param {object} config The settings to be used by the formatter.
 * @returns {array} The final list of messages to be displayed in TeamCity
 */
module.exports = (results, config) => {
  const { reportName } = config;
  const outputList = [];
  let errorCount = 0;
  let warningCount = 0;

  outputList.push(`##teamcity[testSuiteStarted name='${reportName}']`);

  results.forEach(result => {
    const { messages } = result;

    if (messages.length === 0) {
      return;
    }

    const filePath = utils.escapeTeamCityString(result.filePath);

    outputList.push(`##teamcity[testStarted name='${reportName}: ${filePath}']`);

    const errorsList = [];
    const warningsList = [];

    messages.forEach(messageObj => {
      const { line, column, message, ruleId, fatal, severity } = messageObj;

      const formattedMessage = `line ${line}, col ${column}, ${message} (${ruleId})`;

      const isError = fatal || severity === 2;
      if (!isError) {
        warningsList.push(formattedMessage);
        warningCount += 1;
      } else {
        errorsList.push(formattedMessage);
        errorCount += 1;
      }
    });

    // Group errors and warnings together per file
    if (errorsList.length) {
      const errors = utils.escapeTeamCityString(errorsList.join('\n'));
      outputList.push(
        `##teamcity[testFailed name='${reportName}: ${filePath}' message='${errors}']`
      );
    }

    if (warningsList.length) {
      const warnings = utils.escapeTeamCityString(warningsList.join('\n'));
      outputList.push(
        `##teamcity[testStdOut name='${reportName}: ${filePath}' out='warning: ${warnings}']`
      );
    }

    outputList.push(`##teamcity[testFinished name='${reportName}: ${filePath}']`);
  });

  outputList.push(`##teamcity[testSuiteFinished name='${reportName}']`);

  outputList.push(
    `##teamcity[buildStatisticValue key='${config.errorStatisticsName}' value='${errorCount}']`
  );
  outputList.push(
    `##teamcity[buildStatisticValue key='${config.warningStatisticsName}' value='${warningCount}']`
  );

  return outputList;
};
